<?php

namespace LakeDrops\Ahoy;

use Composer\Composer;
use Composer\IO\IOInterface;
use Composer\Script\Event;
use LakeDrops\Component\Composer\Utils;
use Symfony\Component\Yaml\Yaml;

/**
 * Class Handler.
 *
 * @package LakeDrops\Drupal8Scaffold
 */
class Handler {

  /**
   * The composer object running this session.
   *
   * @var \Composer\Composer
   */
  protected $composer;

  /**
   * The input-output object for the composer session.
   *
   * @var \Composer\IO\IOInterface
   */
  protected $io;

  /**
   * Handler constructor.
   *
   * @param \Composer\Composer $composer
   *   The composer object.
   * @param \Composer\IO\IOInterface $io
   *   The input-output object.
   */
  public function __construct(Composer $composer, IOInterface $io) {
    $this->composer = $composer;
    $this->io = $io;
  }

  /**
   * Retrieve a package from the current composer process.
   *
   * @param string $name
   *   Name of the package to get from the current composer installation.
   *
   * @return \Composer\Package\PackageInterface
   *   The package.
   */
  protected function getPackage($name) {
    return $this->composer->getRepositoryManager()->getLocalRepository()->findPackage($name, '*');
  }

  /**
   * Update ahoy scripts of all LakeDrops plugins.
   *
   * @param \Composer\Script\Event $event
   *   The event that triggered the plugin.
   */
  public function updateScripts(Event $event) {

    // We only do the fancy stuff for developers.
    if (!$event->isDevMode()) {
      return;
    }

    $ahoy = [
      'ahoyapi' => 'v2',
      'commands' => [],
    ];
    $installationManager = $this->composer->getInstallationManager();
    foreach ($this->composer->getRepositoryManager()->getLocalRepository()->search('lakedrops/*') as $pkg) {
      $pluginRoot = $installationManager->getInstallPath($this->getPackage($pkg['name']));
      $pluginSettings = new Utils($this->composer, $pluginRoot);
      foreach ($pluginSettings->getSubSubSection('extra', 'lakedrops', 'ahoy') as $command => $commands) {
        $ahoy['commands'][$command] = $commands;
        foreach ($commands['imports'] as $key => $import) {
          $ahoy['commands'][$command]['imports'][$key] = './vendor/' . $pkg['name'] . '/' . $import;
        }
      }
    }
    $file = getcwd() . '/.ahoy.yml';
    file_put_contents($file, $rendered = Yaml::dump($ahoy, 9, 2));
    $this->git('ignore ' . '.ahoy.yml');
  }

  /**
   * Wrapper for git command in the root directory.
   *
   * @param string $command
   *   Git command name, arguments and/or options.
   */
  protected function git($command) {
    passthru(escapeshellcmd('git -c "user.email=d8-project@lakedrops.com" ' . $command));
  }

}
